// FRAGMENTS.CPP
// Code fragments from WavePatch 1.2
// Copyright  1994 Turtle Beach Systems, Inc. All Rights Reserved

// When we are re-activated, tell the document to re-sync with the synthesizer    
void CMainFrame::OnActivateApp(BOOL bActive, HTASK hTask)
{
    CMDIFrameWnd::OnActivateApp(bActive, hTask); 
                                                    
    CMDIChildWnd* pActiveFrame = MDIGetActive();
    if (pActiveFrame != NULL)
    {
        if (bActive)   
        {
            CWaveFrontBankDoc* pBank = (CWaveFrontBankDoc*)pActiveFrame->GetActiveDocument(); 
            ASSERT(pBank != NULL);
    
            pBank->SyncWithSynth();
        }
    }
}

// If some other app has accessed the Gatekeeper, get everything from it.
void CWaveFrontBankDoc::SyncWithSynth()
{
    HTASK hTask = GetCurrentTask();

    // Make sure the program in the synth is up-to-date; 
    // some other app may monkey with it!
    if (m_nCurrentProgram != NO_PROGRAM)
        SendMidiProgramChange(m_nCurrentProgram);
    
    // See if someone else has used the Gatekeeper
    if (GateGetOwner() != hTask)
    {   
        // First and foremost - tell the Gatekeeper who we are.
        CMainFrame* pMainFrame = (CMainFrame*)(AfxGetApp()->m_pMainWnd);
        GateSetOwner(hTask, pMainFrame->m_hWnd);
        
        // Set up audition the way we like it
        SetAuditionInput(m_nAuditionInput);
        GateEnableAudition(TRUE);            
        
        // Get all data from the synth
        GetEverythingFromSynth();
    }
}
                         
// Get all dirty data from the synth
void CWaveFrontBankDoc::GetEverythingFromSynth()
{   
    CMainFrame* pMainFrame = (CMainFrame*)(AfxGetApp()->m_pMainWnd);
    int nPrograms[NUM_MIDICHANNELS];
    CString strProgramName;
    int nProgressCounter = 0;
    int nProgressTotal = NUM_PROGRAMS + NUM_PATCHES + NUM_SAMPLES;     
    
    BeginWaitCursor();  
    
    // Open the synth if necessary. Some drivers may require this
    // (especially SoundBlaster)
    m_hMidiOut = GateOpenSynth(m_nCurrentSynth);
    
    // Audition must be disabled during these operations because... well,
    // it just seems safer that way.
    GateEnableAudition(FALSE);
        
    // Put up a message
    pMainFrame->SetStatusMessage(IDS_GETTINGEVERYTHING);
        
    // First the programs                                    
    for (int i = 0; i < NUM_PROGRAMS; i++)
    {   
        if (GateIsProgramDirty(m_nCurrentSynth, i))
            GetProgramFromSynth(i);
        pMainFrame->SetProgressBar(nProgressCounter++, nProgressTotal);
    }
        
    // Then the patches. This _must_ be done after programs are retrieved
    for (i = 0; i < NUM_PATCHES; i++) 
    {
        if (GateIsPatchDirty(m_nCurrentSynth, i))
            GetPatchFromSynth(i);
        pMainFrame->SetProgressBar(nProgressCounter++, nProgressTotal);
    }
                                                         
    // Then the samples. 
    for (i = 0; i < NUM_SAMPLES; i++) 
    {
        if (GateIsSampleDirty(m_nCurrentSynth, i))
            GetSampleFromSynth(i);
        pMainFrame->SetProgressBar(nProgressCounter++, nProgressTotal);
    }
        
    // The master controls and drum kit
    GetMasterTuning();          // see below
    m_nVolume = GateGetSynthVolume(m_nCurrentSynth); 
    GetDrumChannelFromSynth();  // see below  
    GateGetDrumkit(m_nCurrentSynth, m_pDrumkit);
    
    // Find out if the synth has reverb and/or filters
    m_bHasEffects = GateDoesSynthHaveEffects(m_nCurrentSynth);
    m_bHasFilters = GateDoesSynthHaveFilters(m_nCurrentSynth);
    
    // If the synth has reverb, get all reverb parameters
    if (m_bHasEffects)
        GetReverbFromSynth();
            
    // Get the name of the currently selected program on the audition channel
    // then put it in the status bar
    GetChannelPrograms(nPrograms);
    strProgramName = GetProgramName(nPrograms[m_nAuditionChannel-1]);
    pMainFrame->SetProgressBar(0, nProgressTotal);
    pMainFrame->SetProgramName(strProgramName);
    
    GateEnableAudition(TRUE);
        
    // Finally, get the bank name
    char szBankFilespec[_MAX_PATH];
    GateGetBankFilespec(m_nCurrentSynth, szBankFilespec);
    if (szBankFilespec[0] != 0)
    {
        SetPathName(szBankFilespec, FALSE);
        pMainFrame->OnUpdateFrameTitle(TRUE);
    }
    
    // Restore the "idle" message and the normal cursor
    pMainFrame->SetStatusMessage(AFX_IDS_IDLEMESSAGE);  
    EndWaitCursor();
}
                                                 
// Get the drum channel from the synth. We assume here that there is no more than one channel 
// assigned to drums (even though the synth lets you do an arbitrary number - why??) 
// If no channel is assigned, we return 0 (which means "none"); if more than one is assigned, 
// we use only the lowest one (or 10, if that is assigned).
void CWaveFrontBankDoc::GetDrumChannelFromSynth()
{                  
    CHANNEL_PROGRAMS channelPrograms;
    static const int nMidiChannels[NUM_MIDICHANNELS] = 
    {
        10, 1, 2, 3, 4, 5, 6, 7, 8, 9, 11, 12, 13, 14, 15, 16
    };  
    
    // Get the channel programs from the synthesizer
    GateReportChannelPrograms(m_nCurrentSynth, &channelPrograms);
                                      
    m_nDrumChannel = 0;                         
    for (int i = 0; i < NUM_MIDICHANNELS; i++)
    {   
        // If the program number is greater that MAX_MIDIDATA, it is a drum channel.
        if (channelPrograms.byProgram[nMidiChannels[i] - 1] > MAX_MIDIDATA)
        {                            
            // If we have not already set the drum channel, do so. If it has been set,
            // fix it in the synth by setting the channel to normal status 
            // and setting its program to 0.
            if (m_nDrumChannel == 0)
                m_nDrumChannel = nMidiChannels[i];
            else
            {
                GateSetDrumChannelStatus(m_nCurrentSynth, nMidiChannels[i], FALSE);                 
                SetChannelProgram(nMidiChannels[i], 0);
            }
        }
    }                       
} 

// Get the master tuning and convert it from WaveFront units to cents
int CWaveFrontBankDoc::GetMasterTuning()
{   
    int nTuningReceived = GateGetTuning(m_nCurrentSynth);

    m_nTuning = MulDiv(nTuningReceived, 1200, 2048);
    if (m_nTuning < MIN_UI_TUNING || m_nTuning > MAX_UI_TUNING)
    {
        m_nTuning = 0;
        GateSetTuning(m_nCurrentSynth, 0);
    }
    return(m_nTuning);
}  


// Save a file
void CWaveFrontBankDoc::OnFileSaveAs()
{
    CSaveFileDialog fileDialog(AfxGetApp()->m_pMainWnd, szBankExt); 
    CString strFilespec;
    
    // Put the file name in a buffer and pass that to the open file structure
    strFilespec = GetPathName();
    fileDialog.m_ofn.lpstrFile = strFilespec.GetBuffer(_MAX_PATH);
    
    // If the file was marked as embedded when it was opened, it
    // must be saved the same way.
    fileDialog.m_bEmbeddedWaves = GateIsBankEmbedded(m_nCurrentSynth);
    fileDialog.m_bMakeBackup = FALSE;
    
    if (fileDialog.DoModal() == IDOK)
    {   
        strFilespec.ReleaseBuffer();
        
        m_bEmbeddedWaves = fileDialog.m_bEmbeddedWaves;                 
        m_bMakeBackup    = fileDialog.m_bMakeBackup;                 
        
        if (OnSaveDocument(strFilespec))
        {               
            // Update the bank name in all windows
            CMainFrame* pMainFrame = (CMainFrame*)(AfxGetApp()->m_pMainWnd);
            SetPathName(strFilespec);    
            pMainFrame->OnUpdateFrameTitle(TRUE);
            UpdateAllViews(NULL); 
            
            // Make the file read-only if specified.
            if (fileDialog.GetReadOnlyPref())
            {                           
                CFileStatus fileStatus;
                
                CFile::GetStatus(strFilespec, fileStatus);
                if ((fileStatus.m_attribute & CFile::readOnly) != CFile::readOnly)
                {
                    fileStatus.m_attribute |= CFile::readOnly;
                    CFile::SetStatus(strFilespec, fileStatus); 
                    TRACE("File '%s' is now read-only\n", (LPCSTR)strFilespec);
                }
            }
        }
    }
    else
        strFilespec.ReleaseBuffer();                 
}

// Load a single program
void CWaveFrontBankDoc::LoadProgram(CString strFilespec, int nProgram)
{
    BeginWaitCursor();
        
    if (GateOpenProgramFile(m_nCurrentSynth, nProgram, strFilespec))
    {
        GetEverythingFromSynth();
        UpdateAllViews(NULL, PROGRAMEDIT_HINT);
        SetModifiedFlag();
    }

    // Restore the "idle" message and the normal cursor
    CMainFrame* pMainFrame = (CMainFrame*)(AfxGetApp()->m_pMainWnd);
    pMainFrame->SetStatusMessage(AFX_IDS_IDLEMESSAGE);  
    EndWaitCursor();
}


CWaveFrontBankDoc::~CWaveFrontBankDoc()
{                                         
    // Shut down the audition input and close all synths. 
    // This is essential! WavePatch "owns" all these handles;
    // if we don't close them, then they will still exist but will be invalid.
    // (This took quite a while to figure out, let me tell you.)
    GateSelectAuditionInput(NO_INPUTSELECTED);
    for (i = 0; i < (int)GateGetWaveFrontSynthCount(); i++)
        GateCloseSynth(i);
}

// handle status message text sent from the Gatekeeper
long CMainFrame::OnStatusMessage(WPARAM wParam, LPARAM lParam)
{
    CString strMessage = (LPCSTR)lParam;
    SetStatusMessage(strMessage);                              
    
    return(0L);
}

// handle progress message sent from the Gatekeeper
long CMainFrame::OnProgressMessage(WPARAM wParam, LPARAM lParam)
{   
    SetProgressBar((int)wParam);

    // When we hit 100%, set the display to 0%
    if (wParam >= 100)
        SetProgressBar(0);
        
    return(0L);
}


                                                 
