// WFGATAPI.H
// Interface API for WaveFront Gatekeeper DLL (WFGATE.DLL)
// 2/1/94 J. Johnson                     
// Copyright  1994 Turtle Beach Systems, Inc. All Rights Reserved

#ifndef WFGATAPI_H
#define WFGATAPI_H

// Structures used by the API. These are exact representations of the
// structures specified in the Mockingbird docs.

struct ENVELOPE
{
    BYTE fAttackTime:7;
    BYTE fUnused1:1;
    BYTE fDecay1Time:7;
    BYTE fUnused2:1;
    BYTE fDecay2Time:7;
    BYTE fUnused3:1;
    BYTE fSustainTime:7;
    BYTE fUnused4:1;
    BYTE fReleaseTime:7;
    BYTE fUnused5:1;
    BYTE fRelease2Time:7;
    BYTE fUnused6:1;
    char cAttackLevel;
    char cDecay1Level;
    char cDecay2Level;
    char cSustainLevel;
    char cReleaseLevel;
    BYTE fAttackVelocity:7;
    BYTE fUnused7:1;
    BYTE fVolumeVelocity:7;
    BYTE fUnused8:1;
    BYTE fKeyScale:7;
    BYTE fUnused9:1;
};

struct LFO
{
    BYTE bySampleNumber;
    BYTE fFrequency:7;
    BYTE fUnused1:1;
    BYTE fAMSource:4;
    BYTE fFMSource:4;
    char cFMAmount;
    char cAMAmount;
    char cStartLevel;
    char cEndLevel;
    BYTE fDelayTime:7;
    BYTE fWaveRestart:1;
    BYTE fRampTime:7;
    BYTE fUnused2:1;
};

struct PATCH
{
    int  nFreqBias;         // ** THIS IS IN MOTOROLA FORMAT!! **
    BYTE fAmpBias:7;
    BYTE fUnused1:1;
    BYTE fPortamento:7;
    BYTE fUnused2:1;
    BYTE bySampleNumber;
    BYTE fPitchBend:4;
    BYTE fSampleMSB:1;
    BYTE fUnused3:3;
    BYTE fMono:1;
    BYTE fRetrigger:1;
    BYTE fNoHold:1;
    BYTE fRestart:1;
    BYTE fFilterConfig:2;
    BYTE fReuse:1;
    BYTE fResetLfo:1;    
    BYTE fFMSource2:4;
    BYTE fFMSource1:4;   
    char cFMAmount1;
    char cFMAmount2;
    BYTE fAMSource:4;
    BYTE fUnused4:4;
    char cAMAmount;
    BYTE fFC1MSource:4;
    BYTE fFC2MSource:4;
    char cFC1MAmount;
    char cFC1KeyScale;
    char cFC1FreqBias;
    char cFC2MAmount;
    char cFC2KeyScale;
    char cFC2FreqBias;
    BYTE fRandomizerRate:7;
    BYTE fUnused5:1;
    ENVELOPE envelope1;
    ENVELOPE envelope2;
    LFO lfo1;
    LFO lfo2;
};

struct LAYER
{
    BYTE byPatchNumber;
    BYTE fMixLevel:7;
    BYTE fUnmute:1;
    BYTE fSplitPoint:7;
    BYTE fSplitDir:1;
    BYTE fPanModSource:2;
    BYTE fPanModulated:1;
    BYTE fPan:4;
    BYTE fSplitType:1;
};

struct PROGRAM
{
    LAYER layer[NUM_LAYERS];
};

struct SAMPLE_OFFSET
{
    DWORD fFraction:4;
    DWORD fInteger:20;
    DWORD fUnused:8;
};                
     
// Sample types     
enum 
{
    LINEAR_16BIT, WHITE_NOISE, LINEAR_8BIT, MULAW_8BIT
};

struct SAMPLE
{
    SAMPLE_OFFSET sampleStartOffset;
    SAMPLE_OFFSET loopStartOffset;
    SAMPLE_OFFSET loopEndOffset;
    SAMPLE_OFFSET sampleEndOffset;
    int nFrequencyBias;
    BYTE fSampleResolution:2;  // See preceding enum
    BYTE fUnused1:1;
    BYTE fLoop:1;
    BYTE fBidirectional:1;
    BYTE fUnused2:1;
    BYTE fReverse:1;
    BYTE fUnused3:1;
};

struct MULTISAMPLE
{
    int nNumberOfSamples;   // log2 of the number of samples (0->1, ... 7->128, etc)
    int nSampleNumber[NUM_MIDIKEYS];
};

struct ALIAS
{
    int nOriginalSample;
    SAMPLE_OFFSET sampleStartOffset;
    SAMPLE_OFFSET loopStartOffset;
    SAMPLE_OFFSET sampleEndOffset;
    SAMPLE_OFFSET loopEndOffset;
    int  nFrequencyBias;
    BYTE fSampleResolution:2;
    BYTE fUnused1:1;
    BYTE fLoop:1;
    BYTE fBidirectional:1;
    BYTE fUnused2:1;
    BYTE fReverse:1;
    BYTE fUnused3:1; 
};  

struct DRUM
{
    BYTE byPatchNumber;
    BYTE fMixLevel:7;
    BYTE fUnmute:1;
    BYTE fGroup:4;
    BYTE fUnused1:4;
    BYTE fPanModSource:2;
    BYTE fPanModulated:1;
    BYTE fPanAmount:4;
    BYTE fUnused2:1;
};

struct DRUMKIT
{
    DRUM drum[NUM_MIDIKEYS];
};

struct CHANNEL_PROGRAMS
{
    BYTE byProgram[NUM_MIDICHANNELS];
};

typedef PATCH            FAR* LPPATCH;
typedef PROGRAM          FAR* LPPROGRAM;
typedef SAMPLE           FAR* LPSAMPLE;
typedef ALIAS            FAR* LPALIAS;
typedef MULTISAMPLE      FAR* LPMULTISAMPLE;
typedef DRUM             FAR* LPDRUM;
typedef DRUMKIT          FAR* LPDRUMKIT;
typedef CHANNEL_PROGRAMS FAR* LPCHANNEL_PROGRAMS;  

// Various constants used in the API
const UINT NO_INPUTSELECTED = 0xFFFF;   // Use this to disable the audition input

// Errors returned by GateIsSampleDownloadable
const int GATE_ERR_NOPROBLEM     = 0;  // No error
const int GATE_ERR_TOOLARGE      = 1;  // Sample > 1 meg
const int GATE_ERR_NOMEMORY      = 2;  // Not enough free memory
const int GATE_ERR_BADFORMAT     = 3;  // Unsupported format
const int GATE_ERR_ISSTEREO      = 4;  // Stereo file (** NO LONGER USED **)
const int GATE_ERR_NOSLOTS       = 5;  // No empty sample slots   
const int GATE_ERR_NOTWAVE       = 6;  // Not a wave file
const int GATE_ERR_BADRESOLUTION = 7;  // Unsupported resolution   

// Status messages sent to the owner window
// "Status" messages contain 0 in wParam, LPCSTR to the message in lParam
// "Progress" messages contain a number from 0 to 100 in wParam, 0L in lParam
#define GATE_STATUSMESSAGE  "Gatekeeper Status Message"
#define GATE_PROGRESSMESSAGE "Gatekeeper Progress Message" 
    
////////////////////////////////////////////////////////////////////////////
// Gatekeeper function prototypes. See EXPORTS.CPP for full descriptions.    

#ifdef __cplusplus
extern "C"
{
#endif

// Set and get the task that "owns" (is currently using) the DLL
void  FAR PASCAL _export GateSetOwner(HTASK hTask, HWND hWindow);
HTASK FAR PASCAL _export GateGetOwner();
HWND  FAR PASCAL _export GateGetOwnerWindow();

// Determine how many programs are using the DLL
int FAR PASCAL _export GateGetReferenceCount();  

// Initialize the Gatekeeper. This must be the second call made (after GateSetOwner()).
void FAR PASCAL _export GateInitialize();

// Obtain information on WaveFront synths in the system. 
UINT FAR PASCAL _export GateGetWaveFrontSynthCount();
void FAR PASCAL _export GateGetWaveFrontSynthInputCaps(UINT nSynth, LPMIDIINCAPS lpCaps);
void FAR PASCAL _export GateGetWaveFrontSynthOutputCaps(UINT nSynth, LPMIDIOUTCAPS lpCaps);
void FAR PASCAL _export GateGetSynthName(UINT nSynth, LPSTR lpszName);   
BOOL FAR PASCAL _export GateDoesSynthHaveEffects(UINT nSynth);
BOOL FAR PASCAL _export GateDoesSynthHaveFilters(UINT nSynth);       
BOOL FAR PASCAL _export GateIsSynthUndetectable(UINT nSynth);
void FAR PASCAL _export GateGetMinMaxVoices(UINT nSynth, LPINT lpMin, LPINT lpMax);
void FAR PASCAL _export GateGetSynthIniFileName(UINT nSynth, LPSTR lpszFileName);

// Initialize a synthesizer 
HMIDIOUT FAR PASCAL _export GateInitializeSynth(UINT nSynth, LPCSTR pszFilespec); 
BOOL FAR PASCAL _export GateIsSynthInitialized(UINT nSynth);

// Open a synth's MIDI ports
HMIDIOUT FAR PASCAL _export GateOpenSynth(UINT nSynth);

// Close a synth's MIDI ports
void FAR PASCAL _export GateCloseSynth(UINT nSynth);

// Reset a synth and wipe its memory 
void FAR PASCAL _export GateResetSynth(UINT nSynth);

// Select an audition input and rechannelization channel for the system
BOOL FAR PASCAL _export GateSelectAuditionInput(UINT nInputID); 
void FAR PASCAL _export GateSelectAuditionOutput(UINT nSynth); 
void FAR PASCAL _export GateEnableAudition(BOOL bEnable); 
void FAR PASCAL _export GateSetRechannelizeChannel(int nRechannelizeChannel); 

// Check the dirty state of a sample, program, or patch.
BOOL FAR PASCAL _export GateIsSampleDirty(UINT nSynth, int nSample);
BOOL FAR PASCAL _export GateIsProgramDirty(UINT nSynth, int nProgram);
BOOL FAR PASCAL _export GateIsPatchDirty(UINT nSynth, int nPatch);

// Send/receive programs 
void FAR PASCAL _export GateSetProgram(UINT nSynth, int nProgram, LPPROGRAM lpProgram, 
    LPCSTR pszName);
void FAR PASCAL _export GateGetProgram(UINT nSynth, int nProgram, LPPROGRAM lpProgram, 
    LPSTR pszName);
void FAR PASCAL _export GateSetProgramName(UINT nSynth, int nProgram, LPCSTR pszName);
void FAR PASCAL _export GateGetProgramName(UINT nSynth, int nProgram, LPSTR pszName);

// Send/receive patches                                                                      
void FAR PASCAL _export GateSetPatch(UINT nSynth, int nPatch, LPPATCH lpPatch, 
    LPCSTR pszName);
void FAR PASCAL _export GateGetPatch(UINT nSynth, int nPatch, LPPATCH lpPatch, 
    LPSTR pszName);
void FAR PASCAL _export GateSetPatchName(UINT nSynth, int nPatch, LPCSTR pszName);
void FAR PASCAL _export GateGetPatchName(UINT nSynth, int nPatch, LPSTR pszName);

// Patch informational functions, etc.
int  FAR PASCAL _export GateGetPatchUsageCount(UINT nSynth, int nPatch);           
int  FAR PASCAL _export GateFindUnusedPatch(UINT nSynth, int nFirstPatch);

// Send/receive samples 
BOOL FAR PASCAL _export GateLoadSampleFile(UINT nSynth, int nSample, LPCSTR pszFilespec, 
    int nRootKey);
BOOL FAR PASCAL _export GateLoadMockingbirdFile(UINT nSynth, int nSample, 
    LPCSTR pszFilespec, int nRootKey);
void FAR PASCAL _export GateSetSample(UINT nSynth, int nSample, LPSAMPLE lpSample, 
    LPCSTR pszName);
void FAR PASCAL _export GateGetSample(UINT nSynth, int nSample, LPSAMPLE lpSample, 
    LPSTR pszName);
void FAR PASCAL _export GateSetMultisample(UINT nSynth, int, LPMULTISAMPLE, 
    LPCSTR pszName);
void FAR PASCAL _export GateGetMultisample(UINT nSynth, int, LPMULTISAMPLE, 
    LPSTR pszName);
void FAR PASCAL _export GateSetAlias(UINT nSynth, int nAlias, LPALIAS lpAlias, 
    LPCSTR pszName);
void FAR PASCAL _export GateGetAlias(UINT nSynth, int nAlias, LPALIAS lpAlias, 
    LPSTR pszName);
void FAR PASCAL _export GateDeleteSample(UINT nSynth, int nSample);

// Sample informational functions, utilities, etc.
BOOL  FAR PASCAL _export GateIsSampleDownloadable(UINT nSynth, LPCSTR pszFilespec, 
    LPINT pResult);
BOOL  FAR PASCAL _export GateIsMockingbirdFileDownloadable(UINT nSynth, 
    LPCSTR pszFilespec, LPINT pResult);
int   FAR PASCAL _export GateIdentifySample(UINT nSynth, int nSample, 
    LPDWORD pdwSampleSize);
BOOL  FAR PASCAL _export GateIsSampleInternal(UINT nSynth, int nSample);
BOOL  FAR PASCAL _export GateIsSampleOriginalRom(UINT nSynth, int nSample);
BOOL  FAR PASCAL _export GateIsSampleEditable(UINT nSynth, int nSample);
BOOL  FAR PASCAL _export GateIsSampleNameEditable(UINT nSynth, int nSample);
int   FAR PASCAL _export GateGetSampleUsageCount(UINT nSynth, int nSample);
int   FAR PASCAL _export GateGetSampleRelevantUsageCount(UINT nSynth, int nSample);
void  FAR PASCAL _export GateSetSampleName(UINT nSynth, int nSample, LPCSTR pszName);  
void  FAR PASCAL _export GateGetSampleName(UINT nSynth, int nSample, LPSTR pszName);
DWORD FAR PASCAL _export GateGetSampleRate(UINT nSynth, int nSample);
void  FAR PASCAL _export GateGetSampleFilespec(UINT nSynth, int nSample, 
    LPSTR pszFilespec);
int   FAR PASCAL _export GateFindUnusedSample(UINT nSynth, int nFirstSample);
int   FAR PASCAL _export GateFindOtherChannel(UINT nSynth, int nSample);
DWORD FAR PASCAL _export GateGetSampleCount(UINT nSynth, int nSample);

// Drumkit functions                                                      
void FAR PASCAL _export GateSetDrumkit(UINT nSynth, LPDRUMKIT lpDrumkit);
void FAR PASCAL _export GateGetDrumkit(UINT nSynth, LPDRUMKIT lpDrumkit);
void FAR PASCAL _export GateSetDrum(UINT nSynth, int nKey, LPDRUM lpDrum);
void FAR PASCAL _export GateGetDrum(UINT nSynth, int nKey, LPDRUM lpDrum);

// Set & get various master parameters
void FAR PASCAL _export GateSetSynthVolume(UINT nSynth, int nVolume);
int  FAR PASCAL _export GateGetSynthVolume(UINT nSynth);
void FAR PASCAL _export GateSetNumberOfVoices(UINT nSynth, int nVoices);
int  FAR PASCAL _export GateGetNumberOfVoices(UINT nSynth);
void FAR PASCAL _export GateSetTuning(UINT nSynth, int nTuning);
int  FAR PASCAL _export GateGetTuning(UINT nSynth);
void FAR PASCAL _export GateSetDrumChannelStatus(UINT nSynth, int nChannel, BOOL bStatus);
BOOL FAR PASCAL _export GateGetDrumChannelStatus(UINT nSynth, int nChannel);

// Effect editing functions
void FAR PASCAL _export GateSetReverbDecayTime(UINT nSynth, int nDecayTime);
void FAR PASCAL _export GateSetReverbAbsorption(UINT nSynth, int nAbsorption);
void FAR PASCAL _export GateSetReverbRoomSize(UINT nSynth, int nRoomSize);
void FAR PASCAL _export GateSetReverbPredelay(UINT nSynth, int nPredelay);
void FAR PASCAL _export GateSetReverbMix(UINT nSynth, int nMix);
void FAR PASCAL _export GateSetReverbEverything(UINT nSynth, int nMix, 
    int nDecayTime, int nAbsorption, int nRoomSize, int nPredelay);
int  FAR PASCAL _export GateGetReverbDecayTime(UINT nSynth);
int  FAR PASCAL _export GateGetReverbAbsorption(UINT nSynth);
int  FAR PASCAL _export GateGetReverbRoomSize(UINT nSynth);
int  FAR PASCAL _export GateGetReverbPredelay(UINT nSynth);
int  FAR PASCAL _export GateGetReverbMix(UINT nSynth);

// Retrieve system info
WORD  FAR PASCAL _export GateReportFirmwareVersion(UINT nSynth);
WORD  FAR PASCAL _export GateReportHardwareVersion(UINT nSynth);
void  FAR PASCAL _export GateReportConfiguration(UINT nSynth, LPDWORD pdwTotalRAM, 
    LPDWORD pdwTotalROM);
DWORD FAR PASCAL _export GateReportFreeMemory(UINT nSynth);
WORD  FAR PASCAL _export GateReportNumberOfSamples(UINT nSynth);
void  FAR PASCAL _export GateReportChannelPrograms(UINT nSynth, 
    LPCHANNEL_PROGRAMS lpChannelPrograms);
void  FAR PASCAL _export GateReportLevels(UINT nSynth, LPINT lpLeftLevel, 
    LPINT lpRightLevel);
int   FAR PASCAL _export GateReportPeakLevels(UINT nSynth, LPINT lpLeftLevel, 
    LPINT lpRightLevel);

// load and save banks, etc.
BOOL FAR PASCAL _export GateOpenBankFile(UINT nSynth, LPCSTR szFilespec);
BOOL FAR PASCAL _export GateSaveBankFile(UINT nSynth, LPCSTR szFilespec, BOOL bEmbedded, 
    BOOL bBackup);
BOOL FAR PASCAL _export GateOpenProgramFile(UINT nSynth, UINT nProgram, LPCSTR szFilespec);
BOOL FAR PASCAL _export GateSaveProgramFile(UINT nSynth, UINT nProgram, LPCSTR szFilespec, 
    BOOL bEmbedded, BOOL bBackup);
BOOL FAR PASCAL _export GateOpenDrumkitFile(UINT nSynth, LPCSTR szFilespec);
BOOL FAR PASCAL _export GateSaveDrumkitFile(UINT nSynth, LPCSTR szFilespec, BOOL bEmbedded, 
    BOOL bBackup);                               
BOOL FAR PASCAL _export GateWriteRomFiles(UINT nSynth, LPCSTR szFilespec, LPCSTR szMicrocodeFile,
    UINT nProcessor, UINT nRomSize, UINT nMajorRevision, UINT nMinorRevision, 
    LPCSTR lpszPartNumber, LPCSTR lpszCopyright, BOOL bAddChannelPrograms);

// Bank informational functions
void FAR PASCAL _export GateGetBankComment(UINT nSynth, LPSTR szComment);
void FAR PASCAL _export GateSetBankComment(UINT nSynth, LPCSTR szComment);                   
BOOL FAR PASCAL _export GateIsBankEmbedded(UINT nSynth);                    
void FAR PASCAL _export GateGetBankFilespec(UINT nSynth, LPSTR szFilespec); 
int  FAR PASCAL _export GateGetLoadedSampleCount(UINT nSynth);

// Miscellaneous informational functions
BOOL FAR PASCAL _export GateIsSampleEmbedded(UINT nSynth, int nSample);
BOOL FAR PASCAL _export GateIsProgramEmbedded(UINT nSynth, int nProgram);
BOOL FAR PASCAL _export GateIsDrumkitEmbedded(UINT nSynth);


#ifdef __cplusplus
}
#endif

#endif            

